"use strict";
exports.__esModule = true;
var Matrix = (function () {
    function Matrix() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var data = [];
        for (var i = 0; i < args.length; i++) {
            args[i] = __reduce__(args[i]);
            if (!(args[i] instanceof Array)) {
                if (i == 0) {
                    data.push([]);
                }
                data[data.length - 1].push(args[i]);
            }
            else {
                data.push(args[i]);
            }
        }
        data = __reduce__(data);
        this.data = data;
        this.dim = __shape__(data);
    }
    Matrix.prototype.set = function (data) {
        var indices = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            indices[_i - 1] = arguments[_i];
        }
        if (indices) {
            throw new SyntaxError("not implemented");
        }
        return data;
    };
    return Matrix;
}());
exports.Matrix = Matrix;
function transpose(input) {
    if (input instanceof Matrix) {
        return new Matrix(__transpose__(input.data));
    }
    else if (!isNaN(input)) {
        return input;
    }
}
exports.transpose = transpose;
function sum(array) {
    var A = array[0];
    var B = array[1];
    if (!isNaN(A) && !isNaN(B)) {
        return A + B;
    }
    else {
        return new Matrix(__add__(A.data, B.data));
    }
}
exports.sum = sum;
function prod(array) {
    var A = array[0];
    var B = array[1];
    if (!isNaN(A) && !isNaN(B)) {
        return A * B;
    }
    else if (!isNaN(A)) {
        return new Matrix(__multiplyScalar__(B.data, A));
    }
    else if (!isNaN(B)) {
        return new Matrix(__multiplyScalar__(A.data, B));
    }
    else {
        return new Matrix(__multiply__(A.data, B.data));
    }
}
exports.prod = prod;
function __multiplyScalar__(data, s) {
    var result = [];
    if (isNaN(data[0])) {
        for (var i = 0; i < data.length; i++) {
            result.push(__multiplyScalar__(data[i], s));
        }
    }
    else {
        for (var i = 0; i < data.length; i++) {
            result.push(s * data[i]);
        }
    }
    return result;
}
function __dot__(datA, datB) {
    var result = 0;
    for (var i = 0; i < datA.length; i++) {
        result += datA[i] * datB[i];
    }
    return result;
}
function __multiply__(datA, datB) {
    var datBT = __transpose__(datB);
    var a = __shape__(datA), b = __shape__(datBT);
    var result = [];
    for (var i = 0; i < a[0]; i++) {
        result.push([]);
        for (var j = 0; j < b[0]; j++) {
            result[i].push(__dot__(datA[i], datBT[j]));
        }
    }
    return result;
}
function __reduce__(data) {
    if (data instanceof Matrix) {
        data = __reduce__(data.data);
    }
    if (data.length === 1) {
        data = __reduce__(data[0]);
    }
    return data;
}
function __add__(datA, datB) {
    var result = [];
    if (isNaN(datA[0])) {
        for (var i = 0; i < datA.length; i++) {
            result.push(__add__(datA[i], datB[i]));
        }
    }
    else {
        for (var i = 0; i < datA.length; i++) {
            result.push(datA[i] + datB[i]);
        }
    }
    return result;
}
function __transpose__(data) {
    var result = [];
    for (var j = 0; j < data[0].length; j++) {
        result.push([]);
        for (var i = 0; i < data.length; i++) {
            result[j].push(0);
        }
    }
    for (var j = 0; j < data[0].length; j++) {
        for (var i = 0; i < data.length; i++) {
            result[j][i] = data[i][j];
        }
    }
    return result;
}
function __shape__(data) {
    if (isNaN(data[0])) {
        return [data.length].concat(__shape__(data[0]));
    }
    else {
        return [data.length];
    }
}
//# sourceMappingURL=Matrix.js.map