"use strict";
exports.__esModule = true;
var keywords = [
    'for',
    'if',
    'else',
    'elif',
    'while',
    'end',
    'break',
    'function',
];
var type = {
    '=': 'assignment',
    '(': 'bracket',
    ')': 'bracket',
    '[': 'bracket',
    ']': 'bracket',
    '{': 'bracket',
    '}': 'bracket',
    '+': 'operator',
    '-': 'operator',
    '*': 'operator',
    '/': 'operator',
    '\'': 'operator',
    ',': 'delimiter',
    ';': 'delimiter',
    '\n': 'delimiter'
};
function tokenize(input) {
    var index = 0;
    var NUMBERS = /[0-9]/;
    var LETTERS = /[a-z]/i;
    var tokens = [];
    while (index < input.length) {
        var char = input[index];
        if (NUMBERS.test(char) || char == '.' || char == '-') {
            var value = '';
            if (char === '-') {
                value += char;
                char = input[++index];
            }
            var decimals = (char === '.');
            if (decimals) {
                value += '0' + char;
                char = input[++index];
                if (!NUMBERS.test(char)) {
                    throw new SyntaxError;
                }
            }
            while (char && NUMBERS.test(char)) {
                value += char;
                char = input[++index];
                if (char == '.') {
                    if (decimals) {
                        throw new SyntaxError;
                    }
                    else {
                        decimals = true;
                    }
                    value += char;
                    char = input[++index];
                    if (!NUMBERS.test(char)) {
                        throw new SyntaxError;
                    }
                }
            }
            tokens.push({
                type: 'number',
                value: value
            });
            continue;
        }
        else if (type[char]) {
            tokens.push({
                type: type[char],
                value: char
            });
            index++;
            continue;
        }
        else if (/\s/.test(char)) {
            index++;
            continue;
        }
        else if (LETTERS.test(char)) {
            var value = '';
            while (char && LETTERS.test(char)) {
                value += char;
                char = input[++index];
            }
            if (keywords.includes(value)) {
                tokens.push({
                    type: 'keyword',
                    value: value
                });
            }
            else {
                tokens.push({
                    type: 'identifier',
                    value: value
                });
            }
            continue;
        }
        throw new TypeError('Unknown character: ' + char);
    }
    return tokens;
}
exports.tokenize = tokenize;
//# sourceMappingURL=tokenizer.js.map