"use strict";
exports.__esModule = true;
var expression_js_1 = require("../../lang/ast/expression/expression.js");
var primitive_js_1 = require("../../lang/ast/expression/primitive.js");
var block_js_1 = require("../../lang/ast/block/block.js");
var precedence = {
    '+': 1,
    '*': 2,
    '\'': 3
};
var walk = function (stream, parent) {
    var node = null;
    if (stream.token().type === 'keyword' && stream.token().value === 'function') {
        stream.next();
        var eq = walk(stream, null);
        node = new block_js_1.Fun(eq.rhs.identifier.name, eq.lhs, eq.rhs.arguments, []);
        while (stream.token().type !== 'keyword' || stream.token().value !== 'end') {
            var w = walk(stream, null);
            node.script.push(w);
            if (stream.token() && stream.token().type !== 'delimiter') {
                node.script[node.script.length - 1].verbose = true;
            }
            else {
                node.script[node.script.length - 1].verbose = (stream.token() ? stream.token().value === ',' : true);
            }
            if (stream.token() && stream.token().type === 'delimiter') {
                stream.next();
            }
        }
        return node;
    }
    if (stream.token() && stream.token().type === 'bracket' && stream.token().value === '(') {
        stream.next();
        while (stream.token().type !== 'bracket' || stream.token().value !== ')') {
            node = walk(stream, null);
        }
        stream.next();
    }
    if (stream.token() && stream.token().type === 'number') {
        node = new primitive_js_1.Scalar(stream.token().value);
        stream.next();
    }
    if (stream.token() && stream.token().type === 'identifier') {
        node = new expression_js_1.Identifier(stream.token().value);
        stream.next();
    }
    if (stream.token() && stream.token().type === 'bracket' && stream.token().value === '[') {
        stream.next();
        var data = [];
        while (stream.token().type !== 'bracket' || stream.token().value !== ']') {
            if (stream.token().type === 'delimiter' && stream.token().value === ',') {
                stream.next();
                if (stream.token().type === 'bracket' && stream.token().value === '[') {
                    var w = walk(stream, null);
                    for (var i = 0; i < w.data.length; i++) {
                        data[data.length - 1].data.push(w.data[i]);
                    }
                }
            }
            else if (stream.token().type === 'delimiter' && stream.token().value === ';') {
                stream.next();
                data.push([walk(stream, null)]);
            }
            else {
                data.push(walk(stream, null));
            }
        }
        node = new primitive_js_1.Matrix(data);
        stream.next();
    }
    if (stream.token() && stream.token().type === 'bracket' && stream.token().value === '(') {
        stream.next();
        var nodes = [];
        while (stream.token().type !== 'bracket' || stream.token().value !== ')') {
            nodes.push(walk(stream, null));
            if (stream.token().type === 'delimiter' && stream.token().value === ',') {
                stream.next();
            }
        }
        node = new expression_js_1.Application(node, nodes);
        stream.next();
    }
    while (stream.token() && stream.token().type === 'operator') {
        var op = stream.token().value;
        if (parent && parent instanceof expression_js_1.Operation && precedence[parent.operator] > precedence[op]) {
            return node;
        }
        stream.next();
        node = new expression_js_1.Operation(op, [node,]);
        if (op !== '\'') {
            var w = walk(stream, node);
            node.arguments.push(w);
        }
    }
    if (stream.token() && stream.token().type === 'assignment') {
        stream.next();
        node = new expression_js_1.Assignment(node, walk(stream, null));
    }
    if (!node) {
        throw new SyntaxError();
    }
    return node;
};
function parse(tokens) {
    var stream = {
        index: 0,
        token: function () {
            return tokens[this.index];
        },
        next: function () {
            var token;
            do {
                token = tokens[++this.index];
            } while (token && token.value === '\n');
            return token;
        },
        previous: function () {
            return tokens[this.index - 1];
        }
    };
    var ast = new block_js_1.Block([]);
    do {
        while (stream.token() && stream.token().value === '\n') {
            stream.next();
        }
        if (!stream.token()) {
            break;
        }
        ast.script.push(walk(stream, null));
        if (stream.token() && stream.token().type !== 'delimiter') {
        }
        else if (!(ast.script[ast.script.length - 1] instanceof block_js_1.Block)) {
            ast.script[ast.script.length - 1].verbose = (stream.token() ? stream.token().value === ',' : true);
        }
    } while (stream.next());
    return ast;
}
exports.parse = parse;
//# sourceMappingURL=parser.js.map