"use strict";
exports.__esModule = true;
var block_js_1 = require("../../front/parser/ast/block/block.js");
var expression_js_1 = require("../../front/parser/ast/expression/expression.js");
var primitive_js_1 = require("../../front/parser/ast/expression/primitive.js");
;
function init(node, vars) {
    if (vars === void 0) { vars = []; }
    if (node instanceof block_js_1.Fun) {
        var script = node.script;
        var newVars = [];
        for (var i = 0; i < script.length; i++) {
            var expression = script[i];
            if (expression instanceof expression_js_1.Assignment) {
                var n = init(expression, vars);
                newVars = newVars.concat(n);
            }
        }
        return newVars;
    }
    if (node instanceof block_js_1.Block) {
        var script = node.script;
        var newVars = [];
        for (var i = 0; i < script.length; i++) {
            var expression = script[i];
            if (expression instanceof expression_js_1.Assignment) {
                var n = init(expression, vars);
                newVars = newVars.concat(n);
            }
        }
        return newVars;
    }
    if (node instanceof expression_js_1.Identifier) {
        if (!vars.includes(node.name)) {
            return [node.name];
        }
    }
    if (node instanceof expression_js_1.Assignment) {
        return init(node.lhs, vars);
    }
    if (node instanceof expression_js_1.Application) {
        return init(node.identifier, vars);
    }
    return [];
}
function generate(node, vars, mod) {
    if (vars === void 0) { vars = []; }
    if (mod === void 0) { mod = false; }
    var string = "";
    if (node instanceof block_js_1.Fun) {
        if (mod) {
            string += "import { Matrix, sum, prod, transpose } from '../node_modules/matscript/lib/core/Matrix.js;'\n";
            string += "import { Fun } from '../node_modules/matscript/lib/core/Fun.js;'\n\n";
        }
        vars = init(node);
        var inputs = "";
        for (var i = 0; i < node.input.length; i++) {
            inputs += node.input[i].name;
            if (i + 1 < node.input.length) {
                inputs += ", ";
            }
        }
        if (mod) {
            string += "export function " + node.name + "( " + inputs + " ) {\n\n";
        }
        else {
            string += "let " + node.name + " = new Fun(( " + inputs + " ) => {\n\n";
        }
        for (var i = 0; i < vars.length; i++) {
            string += "let " + vars[i] + ";\n";
        }
        if (vars.length > 0) {
            string += "\n";
        }
        var script = node.script;
        for (var i = 0; i < script.length; i++) {
            var expression = script[i];
            var x = generate(expression, vars);
            if (expression.verbose) {
                string += "console.log(" + x + ");\n";
            }
            else {
                string += x + ';\n';
            }
        }
        string += "\nreturn " + node.output.name + ";\n";
        if (mod) {
            string += "\n};\n";
        }
        else {
            string += "\n} );\n";
        }
    }
    else if (node instanceof block_js_1.Block) {
        vars = vars.concat(init(node));
        for (var i = 0; i < vars.length; i++) {
            string += "let " + vars[i] + ";\n";
        }
        if (vars.length > 0) {
            string += "\n";
        }
        var script = node.script;
        for (var i = 0; i < script.length; i++) {
            var element = script[i];
            var x = generate(element, vars, mod);
            if (element instanceof expression_js_1.Expression && element.verbose) {
                string += "console.log(" + x + ");\n";
            }
            else if (!(element instanceof block_js_1.Block)) {
                string += x + ';\n';
            }
            else {
                string += x + '\n';
            }
        }
        string += "\n";
    }
    if (node instanceof expression_js_1.Assignment) {
        if (node.lhs instanceof expression_js_1.Application) {
            var application = node.lhs;
            var id = generate(application.identifier, vars, mod);
            var content = generate(node.rhs, vars, mod);
            string += "(" + id + " ? " + id + ".set(" + content + ", ";
            var indices = "";
            for (var i = 0; i < application.arguments.length; i++) {
                indices += generate(application.arguments[i], vars);
                if (i < application.arguments.length - 1) {
                    indices += ", ";
                }
            }
            var alternative = "new Matrix([]).set(" + content + ", " + indices + ")";
            string += indices + ") : " + alternative + ")";
        }
        else {
            string += generate(node.lhs, vars, mod) + " = " + generate(node.rhs, vars, mod);
        }
    }
    if (node instanceof expression_js_1.Application) {
        string += generate(node.identifier, vars, mod) + ".apply(";
        for (var i = 0; i < node.arguments.length; i++) {
            string += generate(node.arguments[i], vars, mod);
            if (i < node.arguments.length - 1) {
                string += ", ";
            }
        }
        string += ")";
    }
    if (node instanceof expression_js_1.Identifier) {
        string += node.name;
    }
    if (node instanceof primitive_js_1.Scalar) {
        string += node.data;
    }
    if (node instanceof expression_js_1.Operation) {
        if (node.operator === "'") {
            string += "transpose(" + generate(node.arguments[0], vars, mod) + ")";
        }
        else if (node.operator === "*") {
            string += "prod([" + generate(node.arguments[0], vars, mod) + ", " + generate(node.arguments[1], vars, mod) + "])";
        }
        else if (node.operator === "+") {
            string += "sum([" + generate(node.arguments[0], vars, mod) + ", " + generate(node.arguments[1], vars, mod) + "])";
        }
        else {
            string += generate(node.arguments[0], vars, mod) + " " + node.operator + " " + generate(node.arguments[1], vars, mod);
        }
    }
    if (node instanceof primitive_js_1.Matrix) {
        string += "new Matrix(";
        for (var i = 0; i < node.data.length; i++) {
            string += generate(node.data[i], vars, mod);
            if (i < node.data.length - 1) {
                string += ", ";
            }
        }
        string += ")";
    }
    if (node instanceof Array) {
        string += "new Matrix(";
        for (var i = 0; i < node.length; i++) {
            string += generate(node[i], vars, mod);
            if (i < node.length - 1) {
                string += ", ";
            }
        }
        string += ")";
    }
    return string;
}
exports.generate = generate;
//# sourceMappingURL=generator.js.map